@@ -245,10 +245,10 @@ typedef struct {
 typedef enum handshake_state_t { STATE0 = 0, STATE1, STATE2,
 	STATE3, STATE4, STATE5, STATE6, STATE7, STATE8,
 	STATE9, STATE10, STATE11, STATE12, STATE13, STATE14,
-	STATE15, STATE16, STATE17,
+	STATE15, STATE16, STATE17, STATE18,
 	STATE20 = 20, STATE21, STATE22,
 	STATE30 = 30, STATE31, STATE40 = 40, STATE41, STATE50 = 50,
-	STATE60 = 60, STATE61, STATE62, STATE70, STATE71
+	STATE60 = 60, STATE61, STATE62, 
 } handshake_state_t;
 
 typedef enum heartbeat_state_t {
