@@ -1544,8 +1544,7 @@ gnutls_certificate_set_x509_trust(gnutls_certificate_credentials_t res,
  * will import all certificates that are marked as trusted. Note
  * that the supported URLs are the ones indicated by gnutls_url_is_supported().
  *
- * Returns: number of certificates processed, or a negative error code on
- * error.
+ * Returns: the number of certificates processed
  **/
 int
 gnutls_certificate_set_x509_trust_file(gnutls_certificate_credentials_t
@@ -1562,6 +1561,37 @@ int ret;
 	return ret;
 }
 
+/**
+ * gnutls_certificate_set_x509_trust_dir:
+ * @cred: is a #gnutls_certificate_credentials_t structure.
+ * @ca_dir: is a directory containing the list of trusted CAs (DER or PEM list)
+ * @type: is PEM or DER
+ *
+ * This function adds the trusted CAs present in the directory in order to 
+ * verify client or server certificates. This function is identical
+ * to gnutls_certificate_set_x509_trust_file() but loads all certificates
+ * in a directory.
+ *
+ * Returns: the number of certificates processed
+ *
+ * Since: 3.3.6
+ *
+ **/
+int
+gnutls_certificate_set_x509_trust_dir(gnutls_certificate_credentials_t cred,
+				      const char *ca_dir,
+				      gnutls_x509_crt_fmt_t type)
+{
+int ret;
+
+	ret = gnutls_x509_trust_list_add_trust_dir(cred->tlist, ca_dir, NULL, 
+						type, GNUTLS_TL_USE_IN_TLS, 0);
+	if (ret == GNUTLS_E_NO_CERTIFICATE_FOUND)
+		return 0;
+
+	return ret;
+}
+
 /**
  * gnutls_certificate_set_x509_system_trust:
  * @cred: is a #gnutls_certificate_credentials_t structure.
