@@ -243,6 +243,11 @@ pkcs11_add_module(const char* name, struct ck_function_list *module, const char
 	return 0;
 }
 
+/* Returns:
+ *  - negative error code on error,
+ *  - 0 on success
+ *  - 1 on success (and a fork was detected)
+ */
 int _gnutls_pkcs11_check_init(void)
 {
 	int ret;
@@ -262,6 +267,8 @@ int _gnutls_pkcs11_check_init(void)
 		if (init_pid != getpid()) {
 			/* if we are initialized but a fork is detected */
 			ret = gnutls_pkcs11_reinit();
+			if (ret == 0)
+				ret = 1;
 		}
 #endif
 
@@ -639,13 +646,13 @@ int gnutls_pkcs11_reinit(void)
 		if (providers[i].module != NULL) {
 			rv = p11_kit_module_initialize(providers
 						       [i].module);
-			if (rv != CKR_OK) {
+			if (rv == CKR_OK || rv == CKR_CRYPTOKI_ALREADY_INITIALIZED) {
 				providers[i].active = 1;
 			} else {
 				providers[i].active = 0;
 				_gnutls_debug_log
-				    ("Cannot re-initialize registered module '%s': %s\n",
-				     providers[i].info.library_description,
+				    ("Cannot re-initialize registered module '%.*s': %s\n",
+				     (int)32, providers[i].info.library_description,
 				     p11_kit_strerror(rv));
 			}
 		}
@@ -1280,7 +1287,8 @@ int pkcs11_read_pubkey(struct ck_function_list *module,
 	uint8_t *tmp2 = NULL;
 	size_t tmp1_size, tmp2_size;
 	int ret;
-	
+	ck_rv_t rv;
+
 	tmp1_size = tmp2_size = MAX_PK_PARAM_SIZE;
 	tmp1 = gnutls_malloc(tmp1_size);
 	if (tmp1 == NULL)
@@ -1323,7 +1331,7 @@ int pkcs11_read_pubkey(struct ck_function_list *module,
 		a[1].value = tmp2;
 		a[1].value_len = tmp2_size;
 
-		if (pkcs11_get_attribute_value(module, pks, obj, a, 2) ==
+		if ((rv = pkcs11_get_attribute_value(module, pks, obj, a, 2)) ==
 		    CKR_OK) {
 			ret =
 			    _gnutls_set_datum(&pubkey[0], a[0].value,
@@ -1344,7 +1352,7 @@ int pkcs11_read_pubkey(struct ck_function_list *module,
 			}
 		} else {
 			gnutls_assert();
-			ret = GNUTLS_E_PKCS11_ERROR;
+			ret = pkcs11_rv_to_err(rv);
 			goto cleanup;
 		}
 
@@ -1355,7 +1363,7 @@ int pkcs11_read_pubkey(struct ck_function_list *module,
 		a[1].value = tmp2;
 		a[1].value_len = tmp2_size;
 
-		if (pkcs11_get_attribute_value(module, pks, obj, a, 2) ==
+		if ((rv = pkcs11_get_attribute_value(module, pks, obj, a, 2)) ==
 		    CKR_OK) {
 			pubkey[2].data = a[0].value;
 			pubkey[2].size = a[0].value_len;
@@ -1365,7 +1373,7 @@ int pkcs11_read_pubkey(struct ck_function_list *module,
 
 		} else {
 			gnutls_assert();
-			ret = GNUTLS_E_PKCS11_ERROR;
+			ret = pkcs11_rv_to_err(rv);
 			goto cleanup;
 		}
 		break;
@@ -1373,11 +1381,12 @@ int pkcs11_read_pubkey(struct ck_function_list *module,
 		a[0].type = CKA_EC_PARAMS;
 		a[0].value = tmp1;
 		a[0].value_len = tmp1_size;
+
 		a[1].type = CKA_EC_POINT;
 		a[1].value = tmp2;
 		a[1].value_len = tmp2_size;
 
-		if (pkcs11_get_attribute_value(module, pks, obj, a, 2) ==
+		if ((rv = pkcs11_get_attribute_value(module, pks, obj, a, 2)) ==
 		    CKR_OK) {
 
 			pubkey[0].data = a[0].value;
@@ -1387,12 +1396,14 @@ int pkcs11_read_pubkey(struct ck_function_list *module,
 			pubkey[1].size = a[1].value_len;
 		} else {
 			gnutls_assert();
-			ret = GNUTLS_E_PKCS11_ERROR;
+
+			ret = pkcs11_rv_to_err(rv);
 			goto cleanup;
 		}
 
 		break;
 	default:
+		_gnutls_debug_log("requested reading public key of unsupported type %u\n", (unsigned)key_type);
 		ret = gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
 		goto cleanup;
 	}
@@ -2288,7 +2299,7 @@ find_objs(struct pkcs11_session_info *sinfo,
 	  struct token_info *info, struct ck_info *lib_info, void *input)
 {
 	struct find_obj_data_st *find_data = input;
-	struct ck_attribute a[6];
+	struct ck_attribute a[16];
 	struct ck_attribute *attr;
 	ck_object_class_t class = (ck_object_class_t) - 1;
 	ck_certificate_type_t type = (ck_certificate_type_t) - 1;
