@@ -132,6 +132,89 @@ dane_query_data(dane_query_t q, unsigned int idx,
 	return DANE_E_SUCCESS;
 }
 
+/**
+ * dane_query_to_raw_tlsa:
+ * @q: The query result structure
+ * @data_entries: Pointer set to the number of entries in the query
+ * @dane_data: Pointer to contain an array of DNS rdata items, terminated with a NULL pointer;
+ *             caller must guarantee that the referenced data remains
+ *             valid until dane_query_deinit() is called.
+ * @dane_data_len: Pointer to contain the length n bytes of the dane_data items
+ * @secure: Pointer set true if the result is validated securely, false if
+ *               validation failed or the domain queried has no security info
+ * @bogus: Pointer set true if the result was not secure due to a security failure
+ *
+ * This function will provide the DANE data from the query
+ * response.
+ *
+ * The pointers dane_data and dane_data_len are allocated with gnutls_malloc()
+ * to contain the data from the query result structure (individual
+ * @dane_data items simply point to the original data and are not allocated separately).
+ * The returned @dane_data are only valid during the lifetime of @q.
+ *
+ * Returns: On success, %DANE_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ */
+int
+dane_query_to_raw_tlsa(dane_query_t q, unsigned int *data_entries,
+		char ***dane_data, int **dane_data_len, int *secure, int *bogus)
+{
+	size_t data_sz;
+	char *data_buf;
+	unsigned int idx;
+
+	*data_entries = 0;
+	*dane_data = NULL;
+	*dane_data_len = NULL;
+
+	if (secure) {
+		if (q->status & DANE_QUERY_DNSSEC_VERIFIED)
+			*secure = 1;
+		else
+			*secure = 0;
+	}
+
+	if (bogus) {
+		if (q->status & DANE_QUERY_BOGUS)
+			*bogus = 1;
+		else
+			*bogus = 0;
+	}
+
+	/* pack dane_data pointer list followed by dane_data contents */
+	data_sz = sizeof (**dane_data) * (q->data_entries + 1);
+	for (idx = 0; idx < q->data_entries; idx++)
+		data_sz += 3 + q->data[idx].size;
+
+	*dane_data = gnutls_calloc (1, data_sz);
+	if (*dane_data == NULL)
+		return DANE_E_MEMORY_ERROR;
+	data_buf = (char *)*dane_data;
+	data_buf += sizeof (**dane_data) * (q->data_entries + 1);
+
+	*dane_data_len = gnutls_calloc (q->data_entries + 1, sizeof (**dane_data_len));
+	if (*dane_data_len == NULL) {
+		free(*dane_data);
+		*dane_data = NULL;
+		return DANE_E_MEMORY_ERROR;
+	}
+
+	for (idx = 0; idx < q->data_entries; idx++) {
+		(*dane_data)[idx] = data_buf;
+		(*dane_data)[idx][0] = q->usage[idx];
+		(*dane_data)[idx][1] = q->type[idx];
+		(*dane_data)[idx][2] = q->match[idx];
+		memcpy(&(*dane_data)[idx][3], q->data[idx].data, q->data[idx].size);
+		(*dane_data_len)[idx] = 3 + q->data[idx].size;
+		data_buf += 3 + q->data[idx].size;
+	}
+	(*dane_data)[idx] = NULL;
+	(*dane_data_len)[idx] = 0;
+	*data_entries = q->data_entries;
+
+	return DANE_E_SUCCESS;
+}
+
 /**
  * dane_state_init:
  * @s: The structure to be initialized
@@ -262,7 +345,8 @@ void dane_query_deinit(dane_query_t q)
  *              and the result is due to a security failure, bogus is true.
  *
  * This function will fill in the TLSA (DANE) structure from
- * the given raw DNS record data.
+ * the given raw DNS record data. The @dane_data must be valid
+ * during the lifetime of the query.
  *
  * Returns: On success, %DANE_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
@@ -599,6 +683,22 @@ verify_ee(const gnutls_datum_t * raw_crt,
 	return ret;
 }
 
+#define CHECK_VRET(ret, checked, record_status, status) \
+			if (ret == DANE_E_UNKNOWN_DANE_DATA) { \
+				/* skip that entry */ \
+				continue; \
+			} else if (ret < 0) { \
+				gnutls_assert(); \
+				goto cleanup; \
+			} \
+			checked = 1; \
+			if (record_status == 0) { \
+				status = 0; \
+				break; \
+			} else { \
+				status |= record_status; \
+			}
+
 /**
  * dane_verify_crt_raw:
  * @s: A DANE state structure (may be NULL)
@@ -646,6 +746,9 @@ dane_verify_crt_raw(dane_state_t s,
 	if (chain_type != GNUTLS_CRT_X509)
 		return gnutls_assert_val(DANE_E_INVALID_REQUEST);
 
+	if (chain_size == 0)
+		return gnutls_assert_val(DANE_E_NO_CERT);
+
 	*verify = 0;
 	idx = 0;
 	do {
@@ -668,34 +771,15 @@ dane_verify_crt_raw(dane_state_t s,
 			ret =
 			    verify_ca(chain, chain_size, chain_type, type,
 				      match, &data, &record_verify);
-			if (ret < 0) {
-				gnutls_assert();
-				goto cleanup;
-			}
-			checked = 1;
-			if (record_verify == 0) {
-				*verify = 0;
-				break;
-			} else {
-				*verify |= record_verify;
-			}
+			CHECK_VRET(ret, checked, record_verify, *verify);
+
 		} else if (!(vflags & DANE_VFLAG_ONLY_CHECK_CA_USAGE)
 			   && (usage == DANE_CERT_USAGE_LOCAL_EE
 			       || usage == DANE_CERT_USAGE_EE)) {
 			ret =
 			    verify_ee(&chain[0], chain_type, type, match,
 				      &data, &record_verify);
-			if (ret < 0) {
-				gnutls_assert();
-				goto cleanup;
-			}
-			checked = 1;
-			if (record_verify == 0) {
-				*verify = 0;
-				break;
-			} else {
-				*verify |= record_verify;
-			}
+			CHECK_VRET(ret, checked, record_verify, *verify);
 		}
 	}
 	while (1);
