@@ -978,17 +978,22 @@ pin_callback(void *user, int attempt, const char *token_url,
 	     size_t pin_max)
 {
 	const char *password = NULL;
+	common_info_st *info = user;
 	const char *desc;
 	int cache = MAX_CACHE_TRIES;
 	unsigned len;
 /* allow caching of PIN */
 	static char *cached_url = NULL;
 	static char cached_pin[32] = "";
+	const char *env;
 
-	if (flags & GNUTLS_PIN_SO)
+	if (flags & GNUTLS_PIN_SO) {
+		env = "GNUTLS_SO_PIN";
 		desc = "security officer";
-	else
+	} else {
+		env = "GNUTLS_PIN";
 		desc = "user";
+	}
 
 	if (flags & GNUTLS_PIN_FINAL_TRY) {
 		cache = 0;
@@ -1023,10 +1028,12 @@ pin_callback(void *user, int attempt, const char *token_url,
 
 	printf("Token '%s' with URL '%s' ", token_label, token_url);
 	printf("requires %s PIN\n", desc);
-	
-	password = getenv("GNUTLS_PIN");
 
-	if (password == NULL) {
+	password = getenv(env);
+	if (env == NULL) /* compatibility */
+		password = getenv("GNUTLS_PIN");
+
+	if (password == NULL && (info == NULL || info->batch == 0)) {
 		password = getpass("Enter PIN: ");
 	} else {
 		if (flags & GNUTLS_PIN_WRONG) {
@@ -1036,7 +1043,10 @@ pin_callback(void *user, int attempt, const char *token_url,
 	}
 
 	if (password == NULL || password[0] == 0 || password[0] == '\n') {
-		fprintf(stderr, "No PIN given\n");
+		fprintf(stderr, "No PIN given.\n");
+		if (info != NULL && info->batch != 0) {
+			fprintf(stderr, "note: when operating in batch mode, set the GNUTLS_PIN or GNUTLS_SO_PIN environment variables\n");
+		}
 		exit(1);
 	}
 
@@ -1068,8 +1078,9 @@ static int
 token_callback(void *user, const char *label, const unsigned retry)
 {
 	char buf[32];
+	common_info_st *info = user;
 
-	if (retry > 0) {
+	if (retry > 0 || (info != NULL && info->batch != 0)) {
 		fprintf(stderr, "Could not find token %s\n", label);
 		return -1;
 	}
@@ -1080,11 +1091,11 @@ token_callback(void *user, const char *label, const unsigned retry)
 	return 0;
 }
 
-void pkcs11_common(void)
+void pkcs11_common(common_info_st *c)
 {
 
-	gnutls_pkcs11_set_pin_function(pin_callback, NULL);
-	gnutls_pkcs11_set_token_function(token_callback, NULL);
+	gnutls_pkcs11_set_pin_function(pin_callback, c);
+	gnutls_pkcs11_set_token_function(token_callback, c);
 
 }
 
