@@ -151,6 +151,12 @@ static void cmd_parser(int argc, char **argv)
 	else
 		cinfo.outcert_format = GNUTLS_X509_FMT_PEM;
 
+	if (HAVE_OPT(SET_PIN))
+		cinfo.pin = OPT_ARG(SET_PIN);
+
+	if (HAVE_OPT(SET_SO_PIN))
+		cinfo.so_pin = OPT_ARG(SET_SO_PIN);
+
 	if (HAVE_OPT(LOAD_CERTIFICATE))
 		cinfo.cert = OPT_ARG(LOAD_CERTIFICATE);
 
@@ -174,6 +180,10 @@ static void cmd_parser(int argc, char **argv)
 		bits = OPT_VALUE_BITS;
 	}
 
+	if (HAVE_OPT(CURVE)) {
+		bits = GNUTLS_CURVE_TO_BITS(str_to_curve(OPT_ARG(CURVE)));
+	}
+
 	if (HAVE_OPT(SEC_PARAM)) {
 		sec_param = OPT_ARG(SEC_PARAM);
 	}
@@ -196,7 +206,7 @@ static void cmd_parser(int argc, char **argv)
 	/* handle actions 
 	 */
 	if (HAVE_OPT(LIST_TOKENS))
-		pkcs11_token_list(outfile, detailed_url, &cinfo);
+		pkcs11_token_list(outfile, detailed_url, &cinfo, 0);
 	else if (HAVE_OPT(LIST_MECHANISMS))
 		pkcs11_mechanism_list(outfile, url, login, &cinfo);
 	else if (HAVE_OPT(GENERATE_RANDOM))
@@ -240,7 +250,7 @@ static void cmd_parser(int argc, char **argv)
 	} else if (HAVE_OPT(INITIALIZE))
 		pkcs11_init(outfile, url, label, &cinfo);
 	else if (HAVE_OPT(DELETE))
-		pkcs11_delete(outfile, url, 0, login, &cinfo);
+		pkcs11_delete(outfile, url, login, &cinfo);
 	else if (HAVE_OPT(GENERATE_ECC)) {
 		key_type = GNUTLS_PK_EC;
 		pkcs11_generate(outfile, url, key_type,
