@@ -34,10 +34,17 @@
 #include <stdint.h>
 #include <common.h>
 
-#define FIX(url) \
+static
+char *get_single_token_url(common_info_st * info);
+
+#define FIX(url, out, det, info) \
 	if (url == NULL) { \
-		fprintf(stderr, "warning: no token URL was provided for this operation;\nuse --list-tokens for the available ones.\n"); \
-		exit(1); \
+		url = get_single_token_url(info); \
+		if (url == NULL) { \
+			fprintf(stderr, "warning: no token URL was provided for this operation; the available tokens are:\n"); \
+			pkcs11_token_list(out, det, info, 1); \
+			exit(1); \
+		} \
 	}
 
 #define CHECK_LOGIN_FLAG(flag) \
@@ -46,7 +53,7 @@
 			"warning: --login was not specified and it may be required for this operation.\n")
 
 void
-pkcs11_delete(FILE * outfile, const char *url, int batch,
+pkcs11_delete(FILE * outfile, const char *url,
 	      unsigned int login_flags, common_info_st * info)
 {
 	int ret;
@@ -54,7 +61,7 @@ pkcs11_delete(FILE * outfile, const char *url, int batch,
 
 	if (login_flags) obj_flags = login_flags;
 
-	if (!batch) {
+	if (info->batch == 0) {
 		pkcs11_list(outfile, url, PKCS11_TYPE_ALL, login_flags,
 			    GNUTLS_PKCS11_URL_LIB, info);
 		ret =
@@ -93,9 +100,9 @@ pkcs11_list(FILE * outfile, const char *url, int type, unsigned int login_flags,
 
 	if (login_flags) obj_flags = login_flags;
 
-	pkcs11_common();
+	pkcs11_common(info);
 
-	FIX(url);
+	FIX(url, outfile, detailed, info);
 
 	if (type == PKCS11_TYPE_TRUSTED) {
 		attrs = GNUTLS_PKCS11_OBJ_ATTR_CRT_TRUSTED;
@@ -183,9 +190,9 @@ pkcs11_export(FILE * outfile, const char *url, unsigned int login_flags,
 
 	if (login_flags) obj_flags = login_flags;
 
-	pkcs11_common();
+	pkcs11_common(info);
 
-	FIX(url);
+	FIX(url, outfile, 0, info);
 
 	ret = gnutls_pkcs11_obj_init(&obj);
 	if (ret < 0) {
@@ -231,9 +238,9 @@ pkcs11_export_chain(FILE * outfile, const char *url, unsigned int login_flags,
 
 	if (login_flags) obj_flags = login_flags;
 
-	pkcs11_common();
+	pkcs11_common(info);
 
-	FIX(url);
+	FIX(url, outfile, 0, info);
 
 	ret = gnutls_pkcs11_obj_init(&obj);
 	if (ret < 0) {
@@ -318,9 +325,33 @@ pkcs11_export_chain(FILE * outfile, const char *url, unsigned int login_flags,
 	return;
 }
 
+/* If there is a single token only present, return its URL.
+ */
+static
+char *get_single_token_url(common_info_st * info)
+{
+	int ret;
+	char *url = NULL, *t = NULL;
+
+	pkcs11_common(info);
+
+	ret = gnutls_pkcs11_token_get_url(0, 0, &url);
+	if (ret < 0)
+		return NULL;
+
+	ret = gnutls_pkcs11_token_get_url(1, 0, &t);
+	if (ret != GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+		gnutls_free(url);
+		gnutls_free(t);
+		return NULL;
+	}
+
+	return url;
+}
+
 void
 pkcs11_token_list(FILE * outfile, unsigned int detailed,
-		  common_info_st * info)
+		  common_info_st * info, unsigned brief)
 {
 	int ret;
 	int i;
@@ -328,7 +359,7 @@ pkcs11_token_list(FILE * outfile, unsigned int detailed,
 	char buf[128];
 	size_t size;
 
-	pkcs11_common();
+	pkcs11_common(info);
 
 	for (i = 0;; i++) {
 		ret = gnutls_pkcs11_token_get_url(i, detailed, &url);
@@ -341,7 +372,12 @@ pkcs11_token_list(FILE * outfile, unsigned int detailed,
 			exit(1);
 		}
 
-		fprintf(outfile, "Token %d:\n\tURL: %s\n", i, url);
+		if (brief != 0) {
+			fprintf(outfile, "Token %d: %s\n", i, url);
+			goto cont;
+		} else {
+			fprintf(outfile, "Token %d:\n\tURL: %s\n", i, url);
+		}
 
 		size = sizeof(buf);
 		ret =
@@ -395,7 +431,7 @@ pkcs11_token_list(FILE * outfile, unsigned int detailed,
 
 		fprintf(outfile, "\tSerial: %s\n", buf);
 		fprintf(outfile, "\n\n");
-
+ cont:
 		gnutls_free(url);
 
 	}
@@ -417,11 +453,15 @@ pkcs11_write(FILE * outfile, const char *url, const char *label,
 
 	if (login_flags) flags = login_flags;
 
-	pkcs11_common();
+	pkcs11_common(info);
 
-	FIX(url);
+	FIX(url, outfile, 0, info);
 	CHECK_LOGIN_FLAG(login_flags);
 
+	if (label == NULL && info->batch == 0) {
+		label = read_str("warning: The object's label was not specified.\nLabel: ");
+	}
+
 	secret_key = load_secret_key(0, info);
 	if (secret_key != NULL) {
 		ret =
@@ -445,18 +485,17 @@ pkcs11_write(FILE * outfile, const char *url, const char *label,
 	if (xcrt != NULL) {
 		if (trusted)
 			flags |=
-			    GNUTLS_PKCS11_OBJ_FLAG_MARK_TRUSTED |
-			    GNUTLS_PKCS11_OBJ_FLAG_LOGIN_SO;
+			    GNUTLS_PKCS11_OBJ_FLAG_MARK_TRUSTED;
 
 		if (ca)
 			flags |=
-			    GNUTLS_PKCS11_OBJ_FLAG_MARK_CA |
-			    GNUTLS_PKCS11_OBJ_FLAG_LOGIN_SO;
+			    GNUTLS_PKCS11_OBJ_FLAG_MARK_CA;
 
 		ret = gnutls_pkcs11_copy_x509_crt(url, xcrt, label, flags);
 		if (ret < 0) {
-			fprintf(stderr, "Error in %s:%d: %s\n", __func__,
-				__LINE__, gnutls_strerror(ret));
+			fprintf(stderr, "Error writing certificate: %s\n", gnutls_strerror(ret));
+			if ((ca || trusted) && (flags & GNUTLS_PKCS11_OBJ_FLAG_LOGIN_SO) == 0)
+				fprintf(stderr, "note: some tokens may require security officer login for this operation\n");
 			exit(1);
 		}
 
@@ -498,9 +537,9 @@ pkcs11_generate(FILE * outfile, const char *url, gnutls_pk_algorithm_t pk,
 
 	if (login_flags) flags = login_flags;
 
-	pkcs11_common();
+	pkcs11_common(info);
 
-	FIX(url);
+	FIX(url, outfile, detailed, info);
 	CHECK_LOGIN_FLAG(login_flags);
 
 	if (outfile == stderr || outfile == stdout) {
@@ -509,6 +548,10 @@ pkcs11_generate(FILE * outfile, const char *url, gnutls_pk_algorithm_t pk,
 		sleep(3);
 	}
 
+	if (label == NULL && info->batch == 0) {
+		label = read_str("warning: Label was not specified.\nLabel: ");
+	}
+
 	if (private == 1)
 		flags |= GNUTLS_PKCS11_OBJ_FLAG_MARK_PRIVATE;
 	else if (private == 0)
@@ -521,7 +564,7 @@ pkcs11_generate(FILE * outfile, const char *url, gnutls_pk_algorithm_t pk,
 	if (ret < 0) {
 		fprintf(stderr, "Error in %s:%d: %s\n", __func__, __LINE__,
 			gnutls_strerror(ret));
-		if (bits != 1024)
+		if (bits != 1024 && pk == GNUTLS_PK_RSA)
 			fprintf(stderr,
 				"note: several smart cards do not support arbitrary size keys; try --bits 1024 or 2048.\n");
 		exit(1);
@@ -538,27 +581,42 @@ pkcs11_init(FILE * outfile, const char *url, const char *label,
 	    common_info_st * info)
 {
 	int ret;
-	char *pin;
+	const char *pin;
 	char so_pin[32];
 
-	pkcs11_common();
+	pkcs11_common(info);
 
 	if (url == NULL) {
 		fprintf(stderr, "error: no token URL given to initialize!\n");
 		exit(1);
 	}
 
-	pin = getpass("Enter Security Officer's PIN: ");
-	if (pin == NULL)
-		exit(1);
+	if (info->so_pin != NULL)
+		pin = info->so_pin;
+	else {
+		pin = getenv("GNUTLS_SO_PIN");
+		if (pin == NULL && info->batch == 0)
+			pin = getpass("Enter Security Officer's PIN: ");
+		if (pin == NULL)
+			exit(1);
+	}
 
-	if (strlen(pin) >= sizeof(so_pin))
+	if (strlen(pin) >= sizeof(so_pin) || pin[0] == '\n')
 		exit(1);
 
 	strcpy(so_pin, pin);
 
-	pin = getpass("Enter new User's PIN: ");
-	if (pin == NULL)
+	if (info->so_pin != NULL) {
+		pin = info->pin;
+	} else {
+		pin = getenv("GNUTLS_PIN");
+		if (pin == NULL && info->batch == 0)
+			pin = getpass("Enter new User's PIN: ");
+		if (pin == NULL)
+			exit(1);
+	}
+
+	if (pin[0] == '\n')
 		exit(1);
 
 	ret = gnutls_pkcs11_token_init(url, so_pin, label);
@@ -800,9 +858,9 @@ pkcs11_mechanism_list(FILE * outfile, const char *url, unsigned int login_flags,
 	unsigned long mechanism;
 	const char *str;
 
-	pkcs11_common();
+	pkcs11_common(info);
 
-	FIX(url);
+	FIX(url, outfile, 0, info);
 
 	idx = 0;
 	do {
@@ -833,9 +891,9 @@ pkcs11_get_random(FILE * outfile, const char *url, unsigned bytes,
 	int ret;
 	uint8_t *output;
 
-	pkcs11_common();
+	pkcs11_common(info);
 
-	FIX(url);
+	FIX(url, outfile, 0, info);
 
 	output = malloc(bytes);
 	if (output == NULL) {
